@@ -1,5 +1,5 @@
-use nodejs_resolver::{ResolveResult, Resolver};
-use std::collections::HashSet;
+use nodejs_resolver::{ResolveResult, Resolver, ResolverOptions};
+use std::collections::{HashMap, HashSet};
 use std::path::PathBuf;
 
 macro_rules! get_cases_path {
@@ -52,7 +52,10 @@ macro_rules! vec_to_set {
 #[test]
 fn extensions_test() {
     let extensions_cases_path = get_cases_path!("tests/fixtures/extensions");
-    let resolver = Resolver::default().with_extensions(vec!["ts", "js"]);
+    let resolver = Resolver::new(ResolverOptions {
+        extensions: vec![String::from("ts"), String::from(".js")], // `extensions` can start with `.` or not.
+        ..Default::default()
+    });
 
     should_equal!(resolver, &extensions_cases_path, "./a"; p(vec!["extensions", "a.ts"]));
     should_equal!(resolver, &extensions_cases_path, "./a.js"; p(vec!["extensions", "a.js"]));
@@ -69,15 +72,24 @@ fn extensions_test() {
 #[test]
 fn alias_test() {
     let alias_cases_path = get_cases_path!("tests/fixtures/alias");
-    let resolver = Resolver::default().with_alias(vec![
-        ("aliasA", Some("./a")),
-        ("./b$", Some("./a/index")), // TODO: should we use trailing?
-        ("recursive", Some("./recursive/dir")),
-        ("#", Some("./c/dir")),
-        ("@", Some("./c/dir")),
-        ("@", Some("./c/dir")),
-        ("ignore", None),
-    ]);
+    let resolver = Resolver::new(ResolverOptions {
+        alias: HashMap::from_iter(
+            [
+                (String::from("aliasA"), Some(String::from("./a"))),
+                (String::from("./b$"), Some(String::from("./a/index"))), // TODO: should we use trailing?
+                (
+                    String::from("recursive"),
+                    Some(String::from("./recursive/dir")),
+                ),
+                (String::from("#"), Some(String::from("./c/dir"))),
+                (String::from("@"), Some(String::from("./c/dir"))),
+                (String::from("@"), Some(String::from("./c/dir"))),
+                (String::from("ignore"), None),
+            ]
+            .into_iter(),
+        ),
+        ..Default::default()
+    });
 
     should_equal!(resolver, &alias_cases_path, "./a"; p(vec!["alias", "a", "index"]));
     should_equal!(resolver, &alias_cases_path, "./a/index"; p(vec!["alias", "a", "index"]));
@@ -145,7 +157,10 @@ fn symlink_test() {
     should_equal!(resolver, &symlink_cases_path.join("linked/that/outer/linked"), "./lib/index.js"; p(vec!["symlink", "lib", "index.js"]));
 
     let linked_path = symlink_cases_path.join("linked");
-    let resolver = Resolver::default().with_symlinks(false);
+    let resolver = Resolver::new(ResolverOptions {
+        symlinks: false,
+        ..Default::default()
+    });
 
     should_equal!(resolver, &linked_path, "./index.js"; p(vec!["symlink", "linked", "index.js"]));
     should_equal!(resolver, &linked_path, "./this/this/index.js"; p(vec!["symlink", "linked", "this", "this", "index.js"]));
@@ -209,7 +224,10 @@ fn resolve_test() {
 #[test]
 fn browser_filed_test() {
     let browser_module_case_path = get_cases_path!("tests/fixtures/browser-module");
-    let resolver = Resolver::default().with_alias_fields(vec!["browser"]);
+    let resolver = Resolver::new(ResolverOptions {
+        alias_fields: vec![String::from("browser")],
+        ..Default::default()
+    });
 
     should_ignore!(resolver, &browser_module_case_path, "./lib/ignore");
     should_ignore!(resolver, &browser_module_case_path, "./lib/ignore.js");
@@ -241,10 +259,11 @@ fn browser_filed_test() {
 #[test]
 fn dependencies_test() {
     let dep_case_path = get_cases_path!("tests/fixtures/dependencies");
-    let resolver = Resolver::default()
-        .with_modules(vec!["modules", "node_modules"])
-        .with_extensions(vec![".json", ".js"]);
-
+    let resolver = Resolver::new(ResolverOptions {
+        modules: vec![String::from("modules"), String::from("node_modules")],
+        extensions: vec![String::from(".json"), String::from(".js")],
+        ..Default::default()
+    });
     should_equal!(resolver, &dep_case_path.join("a/b/c"), "module/file"; p(vec!["dependencies", "a", "node_modules", "module", "file.js"]));
     should_equal!(resolver, &dep_case_path.join("a/b/c"), "other-module/file.js"; p(vec!["dependencies", "modules", "other-module", "file.js"]));
 
@@ -256,10 +275,17 @@ fn dependencies_test() {
 fn full_specified_test() {
     // TODO: should I need add `fullSpecified` flag?
     let full_cases_path = get_cases_path!("tests/fixtures/full/a");
-    let resolver = Resolver::default()
-        .with_alias(vec![("alias1", Some("./abc")), ("alias2", Some("./"))])
-        .with_alias_fields(vec!["browser"]);
-
+    let resolver = Resolver::new(ResolverOptions {
+        alias: HashMap::from_iter(
+            [
+                (String::from("alias1"), Some(String::from("./abc"))),
+                (String::from("alias2"), Some(String::from("./"))),
+            ]
+            .into_iter(),
+        ),
+        alias_fields: vec![String::from("browser")],
+        ..Default::default()
+    });
     should_equal!(resolver, &full_cases_path, "./abc.js"; p(vec!["full", "a", "abc.js"]));
     should_equal!(resolver, &full_cases_path, "package1/file.js"; p(vec!["full", "a", "node_modules", "package1", "file.js"]));
     should_equal!(resolver, &full_cases_path, "package1"; p(vec!["full", "a", "node_modules", "package1", "index.js"]));
@@ -304,8 +330,10 @@ fn incorrect_package_test() {
 #[test]
 fn scoped_packages_test() {
     let scoped_path = get_cases_path!("tests/fixtures/scoped");
-    let resolver = Resolver::default().with_alias_fields(vec!["browser"]);
-
+    let resolver = Resolver::new(ResolverOptions {
+        alias_fields: vec![String::from("browser")],
+        ..Default::default()
+    });
     should_equal!(resolver, &scoped_path, "@scope/pack1"; p(vec!["scoped", "node_modules", "@scope", "pack1", "main.js"]));
     should_equal!(resolver, &scoped_path, "@scope/pack1/main"; p(vec!["scoped", "node_modules", "@scope", "pack1", "main.js"]));
     should_equal!(resolver, &scoped_path, "@scope/pack2"; p(vec!["scoped", "node_modules", "@scope", "pack2", "main.js"]));
@@ -318,10 +346,11 @@ fn exports_fields_test() {
     // TODO: [`exports_fields`](https://github.com/webpack/enhanced-resolve/blob/main/test/exportsField.js#L2280) flag
 
     let export_cases_path = get_cases_path!("tests/fixtures/exports-field");
-    let resolver = Resolver::default()
-        .with_extensions(vec![".js"])
-        .with_condition_names(vec_to_set!(["webpack"]));
-
+    let resolver = Resolver::new(ResolverOptions {
+        extensions: vec![String::from("js")],
+        condition_names: vec_to_set!(["webpack"]),
+        ..Default::default()
+    });
     should_error!(resolver, &export_cases_path, "exports-field/dist/../../../a.js"; "Package path exports-field/dist/../../../a.js is not exported");
     should_error!(resolver, &export_cases_path, "exports-field/dist/a.js"; "Package path exports-field/dist/a.js is not exported");
     should_equal!(resolver, &export_cases_path, "exports-field/dist/main.js"; p(vec!["exports-field", "node_modules", "exports-field", "lib", "lib2", "main.js"]));
@@ -333,6 +362,8 @@ fn exports_fields_test() {
     should_error!(resolver, &export_cases_path, "exports-field/dist"; "Package path exports-field/dist is not exported");
     should_error!(resolver, &export_cases_path, "exports-field/lib"; "Package path exports-field/lib is not exported");
     should_error!(resolver, &export_cases_path, "invalid-exports-field"; "Export field key can't mixed relative path and conditional object");
+    should_equal!(resolver, &export_cases_path, "exports-field"; p(vec!["exports-field", "node_modules", "exports-field", "x.js"]));
+
     let export_cases_path2 = get_cases_path!("tests/fixtures/exports-field2");
 
     // TODO: maybe we need provide `full_specified` flag.
@@ -346,24 +377,21 @@ fn exports_fields_test() {
     should_error!(resolver, &export_cases_path2, "exports-field#foo"; "Package path exports-field is not exported");
     should_equal!(resolver, &export_cases_path2, "exports-field/dist/browser.js#foo"; p(vec!["exports-field2", "node_modules", "exports-field", "lib", "browser.js#foo"]));
 
-    let resolver = Resolver::default()
-        .with_extensions(vec![".js"])
-        .with_alias_fields(vec!["browser"])
-        .with_condition_names(vec_to_set!(["webpack"]));
-
+    let resolver = Resolver::new(ResolverOptions {
+        extensions: vec![String::from("js")],
+        alias_fields: vec![String::from("browser")],
+        condition_names: vec_to_set!(["webpack"]),
+        ..Default::default()
+    });
     should_equal!(resolver, &export_cases_path, "./node_modules/exports-field/lib/main.js"; p(vec!["exports-field", "node_modules", "exports-field", "lib", "browser.js"]));
     should_equal!(resolver, &export_cases_path, "./node_modules/exports-field/dist/main.js"; p(vec!["exports-field", "node_modules", "exports-field", "lib", "browser.js"]));
 
-    let resolver = Resolver::default()
-        .with_extensions(vec![".js"])
-        .with_condition_names(vec_to_set!(["webpack"]));
-
-    should_equal!(resolver, &export_cases_path, "exports-field"; p(vec!["exports-field", "node_modules", "exports-field", "x.js"]));
-
-    let resolver = Resolver::default()
-        .with_extensions(vec![".js"])
-        .with_alias_fields(vec!["browser"])
-        .with_condition_names(vec_to_set!(["node"]));
+    let resolver = Resolver::new(ResolverOptions {
+        extensions: vec![String::from(".js")],
+        alias_fields: vec![String::from("browser")],
+        condition_names: vec_to_set!(["node"]),
+        ..Default::default()
+    });
 
     should_equal!(resolver, &export_cases_path, "exports-field/dist/main.js"; p(vec!["exports-field", "node_modules", "exports-field", "lib", "browser.js"]));
 }
@@ -372,10 +400,11 @@ fn exports_fields_test() {
 fn imports_fields_test() {
     // TODO: ['imports_fields`](https://github.com/webpack/enhanced-resolve/blob/main/test/importsField.js#L1228)
     let import_cases_path = get_cases_path!("tests/fixtures/imports-field");
-    let resolver = Resolver::default()
-        .with_extensions(vec![".js"])
-        .with_condition_names(vec_to_set!(["webpack"]));
-
+    let resolver = Resolver::new(ResolverOptions {
+        extensions: vec![String::from(".js")],
+        condition_names: vec_to_set!(["webpack"]),
+        ..Default::default()
+    });
     should_equal!(resolver, &import_cases_path, "#c"; p(vec!["imports-field", "node_modules", "c", "index.js"]));
     should_equal!(resolver, &import_cases_path, "#imports-field"; p(vec!["imports-field", "b.js"]));
     should_equal!(resolver, &import_cases_path, "#b"; p(vec!["b.js"]));
@@ -387,12 +416,25 @@ fn imports_fields_test() {
 }
 
 #[test]
-fn without_description_file() {
+fn without_description_file_test() {
     let fixture_path = p(vec![]);
-    let resolver = Resolver::default()
-        .with_extensions(vec![".js"])
-        .with_description_file(None);
+    let resolver = Resolver::new(ResolverOptions {
+        extensions: vec![String::from(".js")],
+        description_file: None,
+        ..Default::default()
+    });
     should_equal!(resolver, &fixture_path, "./a"; p(vec!["a.js"]));
     let export_cases_path = get_cases_path!("tests/fixtures/exports-field");
     should_equal!(resolver, &export_cases_path, "exports-field/lib"; p(vec!["exports-field", "node_modules","exports-field", "lib", "index.js"]));
 }
+
+#[test]
+fn prefer_relative_test() {
+    let fixture_path = p(vec![]);
+    let resolver = Resolver::new(ResolverOptions {
+        prefer_relative: true,
+        ..Default::default()
+    });
+    should_equal!(resolver, &fixture_path, "main1.js"; p(vec!["main1.js"]));
+    should_equal!(resolver, &fixture_path, "m1/a.js"; p(vec!["node_modules", "m1", "a.js"]));
+}
